unit uniDUTTrashTest;
(*
   Daniel U. Thibault
   <D.U.Thibault@Bigfoot.com>
   revised 02 March 2009 - In StrToSID, replaced shl with shr
                           (the D7 compiler caught this very old mistake)
                           Refactored object names, other tweaks.
    1999-2009 Daniel U. Thibault.  All Rights Reserved.

   This is freeware.  No legally binding promises.  If you use
   any part of this code, you must include an acknowledgement in the
   finished code or executable (in the About box and/or the Help file).
*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus;

type
  TfrmDUTTrashTest = class(TForm)
    lbxFiles: TListBox;
    pnlTrashTest: TPanel;
    btnRefresh: TButton;
    btnRestore: TButton;
    btnTest: TButton;
    btnDelete: TButton;
    mnuPopup: TPopupMenu;
    mnuRefresh: TMenuItem;
    mnuRestore: TMenuItem;
    mnuDelete: TMenuItem;
    mnuCopy: TMenuItem;
    btnCopy: TButton;
    procedure btnRefreshClick(Sender: TObject);
    procedure btnRestoreClick(Sender: TObject);
    procedure btnTestClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure mnuPopupPopup(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmDUTTrashTest: TfrmDUTTrashTest;

implementation

{$R *.DFM}

uses
   Clipbrd,
   DUTUtilsTrash;

const
   //WinNT declarations from WinNT.h
   SID_REVISION                    =  1; // Current revision level
   SID_MAX_SUB_AUTHORITIES         = 15;

procedure TfrmDUTTrashTest.btnRefreshClick(Sender: TObject);
var
   DeletionDateTimes,
   FileSizes : TList;
   FileNames : TStringList;
   i         : Integer;
begin
   Screen.Cursor := crHourGlass;
   pnlTrashTest.Enabled := False;
   DeletionDateTimes := TList.Create;
   FileSizes := TList.Create;
   FileNames := GetTrashContents(DeletionDateTimes, FileSizes);
   lbxFiles.Clear;
   FileNames.Sort;
   lbxFiles.Items.AddStrings(FileNames);
   Caption := 'DUTTrashTest - ' + IntToStr(FileNames.Count) + ' items';
   FileNames.Free;
   for i := 0 to (FileSizes.Count - 1) do begin
      FreeMem(FileSizes[i]);
      FreeMem(DeletionDateTimes[i]);
   end; { for }
   FileSizes.Free;
   DeletionDateTimes.Free;
   pnlTrashTest.Enabled := True;
   btnRestore.Enabled := (lbxFiles.Items.Count > 0);
   btnDelete.Enabled := btnRestore.Enabled;
   btnCopy.Enabled := btnRestore.Enabled;
   Screen.Cursor := crDefault;
end;

procedure TfrmDUTTrashTest.btnRestoreClick(Sender: TObject);
var
   i : Integer;
begin
   if lbxFiles.ItemIndex < 0 then begin
      Beep;
      Exit;
   end; { then }
   for i := 0 to (lbxFiles.Items.Count - 1) do
      if lbxFiles.Selected[i] then RestoreFromTrash(lbxFiles.Items[i]);
   btnRefresh.Click;
end;

function StrToSID(const SIDName : String) : PSID;
{
   Allocates and fills in an SID based on the string SIDName.
   An exception is raised (and no memory allocated) if the
   SID creation fails.
   The SIDName should be of the form
   'S-' + SID_REVISION + '-' + SID_IDENTIFIER_AUTHORITY + ['-' + SIDSubAuthority]*
   The caller is responsible for calling FreeSID to free the PSID.

   We need this function because ConvertStringSIDToSID is only available in Win2000 (NT 5).

   Based on code supplied by
   Colin Wilson <colin@wilsonc.demon.co.uk>
   15 October 1999
}
var
   iSubStr,
   iSubStrTerminator  : Integer;
   ValueStr           : String;
   SIA                : TSIDIdentifierAuthority;
   i,
   AuthorityCount     : DWORD;
   SubAuthority       : Array[1..SID_MAX_SUB_AUTHORITIES] of DWORD;
begin
{
   //SID
   type SID = record
      Revision,
      SubAuthorityCount   : BYTE;
      IdentifierAuthority : TSIDIdentifierAuthority;
      //SID will be shorter if SubAuthority isn't completely filled in
//    SubAuthority        : Array[1..SubAuthorityCount] of DWORD;
      SubAuthority        : Array[1..SID_MAX_SUB_AUTHORITIES] of DWORD;
   end;
   //SID_IDENTIFIER_AUTHORITY
   TSIDIdentifierAuthority = record
      Value: array[0..5] of Byte;
   end;
}
   //SIDName should start with 'S-'
   ValueStr := Copy(SIDName, 1, 2);
   if ValueStr <> 'S-' then raise Exception.Create('SIDToStr: SIDName must begin with "S-"');

   iSubStr := 3;
   //iSubStr points to the beginning of the SID_REVISION substring
   iSubStrTerminator := iSubStr + Pos('-', Copy(SIDName, iSubStr, 9999)) - 1;
   if iSubStrTerminator < iSubStr then raise Exception.Create('SIDToStr: Bad SIDName: Unable to recognise SID_REVISION substring');
   //iSubStrTerminator points to the '-' after the SID_REVISION substring
// ValueStr := Copy(SIDName, iSubStr, iSubStrTerminator - iSubStr);

   iSubStr := iSubStrTerminator + 1;
   //iSubStr now points to the beginning of the IdentifierAuthority substring
   iSubStrTerminator := iSubStr + Pos('-', Copy(SIDName, iSubStr, 9999)) - 1;
   if iSubStrTerminator < iSubStr then raise Exception.Create('SIDToStr: Bad SIDName: Unable to recognise SID_IDENTIFIER_AUTHORITY substring');
   //iSubStrTerminator now points to the '-' after the IdentifierAuthority substring
   ValueStr := Copy(SIDName, iSubStr, iSubStrTerminator - iSubStr);

   //Fill in the SID's IdentifierAuthority
   if ValueStr[1] = '0x' then begin
      { Note: This hex format is undocumented }
      if Length(ValueStr) <> 14 then raise Exception.Create('SIDToStr: Bad SIDName: SID_IDENTIFIER_AUTHORITY incorrectly formatted');
      SIA.Value[0] := StrToInt('$' + Copy(ValueStr,  3, 2));
      SIA.Value[1] := StrToInt('$' + Copy(ValueStr,  5, 2));
      SIA.Value[2] := StrToInt('$' + Copy(ValueStr,  7, 2));
      SIA.Value[3] := StrToInt('$' + Copy(ValueStr,  9, 2));
      SIA.Value[4] := StrToInt('$' + Copy(ValueStr, 11, 2));
      SIA.Value[5] := StrToInt('$' + Copy(ValueStr, 13, 2))
   end else begin
      i := StrToInt(ValueStr);
      SIA.Value[0] := 0;
      SIA.Value[1] := 0;
      SIA.Value[2] := (i shr 24) and $FF;
      SIA.Value[3] := (i shr 16) and $FF;
      SIA.Value[4] := (i shr  8) and $FF;
      SIA.Value[5] :=  i         and $FF;
   end; { else }

   iSubStr := iSubStrTerminator + 1;
   ValueStr := Copy(SIDName, iSubStr, 9999);
   //ValueStr is the SubAuthority[AuthorityCount] substring
   AuthorityCount := 0;
   //iSubStr now points to the SubAuthority[AuthorityCount] substring
   while ValueStr <> '' do begin
      Inc(AuthorityCount);
      iSubStrTerminator := Pos('-', ValueStr);
      if iSubStrTerminator <= 0 then begin
         SubAuthority[AuthorityCount] := StrToInt(ValueStr);
         ValueStr := '';
      end else begin
         //iSubStrTerminator now points to the '-' that follows the SubAuthority[AuthorityCount] substring
//       SubAuthority[AuthorityCount] := StrToCardinal(Copy(ValueStr, 1, iSubStrTerminator - 1));
//       SubAuthority[AuthorityCount] := $FFFFFFFF and StrToInt64(Copy(ValueStr, 1, iSubStrTerminator - 1));
         SubAuthority[AuthorityCount] := StrToInt64(Copy(ValueStr, 1, iSubStrTerminator - 1));
         ValueStr := Copy(ValueStr, iSubStrTerminator + 1, 9999);
      end; { else }
   end; { while }

   if not AllocateAndInitializeSID(
      SIA,
      AuthorityCount,
      SubAuthority[1],
      SubAuthority[2],
      SubAuthority[3],
      SubAuthority[4],
      SubAuthority[5],
      SubAuthority[6],
      SubAuthority[7],
      SubAuthority[8],
      Result
   ) then raise Exception.Create('SIDToStr: Failed to create the SID');

   //Note that there is no guarantee the resulting SID has the same
   //Revision substring as the input SIDName.
end; { StrToSID }

procedure TfrmDUTTrashTest.btnTestClick(Sender: TObject);
const
   testSID = 'S-1-5-21-3840063661-861358421-3458945882-1296';
var
   SID : PSID;
begin
   SID := StrToSID(testSID);
   MessageDlg('StrToSID succeeded', mtCustom, [mbOK], 0);
   MessageDlg('SID ' + testSID + #13#10 + '=  ' + SIDtoStr(SID) + ' ', mtCustom, [mbOK], 0);
   FreeSID(SID);
end;

procedure TfrmDUTTrashTest.btnDeleteClick(Sender: TObject);
var
   i : Integer;
begin
   if lbxFiles.ItemIndex < 0 then begin
      Beep;
      Exit;
   end; { then }
   for i := 0 to (lbxFiles.Items.Count - 1) do
      if lbxFiles.Selected[i] then DeleteFromTrash(lbxFiles.Items[i]);
end;

procedure TfrmDUTTrashTest.btnCopyClick(Sender: TObject);
const
   CRLF = #13#10;
var
   i : Integer;
   S : String;
begin
   if lbxFiles.ItemIndex < 0 then begin
      Beep;
      Exit;
   end; { then }
   S := '';
   for i := 0 to (lbxFiles.Items.Count - 1) do
      if lbxFiles.Selected[i] then S := S + lbxFiles.Items[i] + CRLF;
    Clipboard.SetTextBuf(PChar(S));
end;

procedure TfrmDUTTrashTest.mnuPopupPopup(Sender: TObject);
begin
   mnuRestore.Enabled := (lbxFiles.SelCount > 0);
   mnuDelete.Enabled := mnuRestore.Enabled;
   mnuCopy.Enabled := mnuRestore.Enabled;
end;

end.
